package com.srccodes.examples.howto;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

/**
 * @author Abhijit Ghosh
 * @version 1.0
 *
 */
public class HowToRetainXMLDocType {
	private static final String SAMPLE_XML_FILE = "sampleXml.xml";

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			// Get the xml document
			Document xmlDoc = getDocument(SAMPLE_XML_FILE);
			
			// Modify the 'changefreq' attribute of the element having id 10
			Element articleElement = xmlDoc.getElementById("10");
			articleElement.setAttribute("changefreq", "monthly");
			
			// Update the xml file
			saveToXml(xmlDoc, SAMPLE_XML_FILE);
			
		} catch (Exception ex) { // For simplicity Exception is used instead of handling specific one
			ex.printStackTrace();
		}
	}
	
    /**
     * To get DOM Document from the xml file.
     * 
     * @param filePath
     * @return DOM Document
     * @throws ParserConfigurationException
     * @throws SAXException
     * @throws IOException
     */
    public static Document getDocument(String filePath) throws ParserConfigurationException, SAXException, IOException {
        DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
        
        Document xmlDoc = docBuilder.parse(filePath);
        
        return xmlDoc;
    }
    
    /**
     * To save the Document in xml file 
     * 
     * @param xmlDoc
     * @param filePath
     * @throws TransformerException
     */
    public static void saveToXml(Document xmlDoc, String filePath) throws TransformerException {
        DOMSource source = new DOMSource(xmlDoc);
        StreamResult result = new StreamResult(new File(filePath));
        
        TransformerFactory transformerFactory = TransformerFactory.newInstance();
        Transformer transformer = transformerFactory.newTransformer();
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");  
        
        DocumentType doctype = xmlDoc.getDoctype();
        if(doctype != null) {
        	transformer.setOutputProperty(OutputKeys.DOCTYPE_PUBLIC, doctype.getPublicId());
        	transformer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM, doctype.getSystemId());
        }
        
        transformer.transform(source, result);
    }
}
